# -*- coding: UTF-8 -*-
"""
Software: LingFeat - Comprehensive Linguistic Features for Readability Assessment
Page: OSKF.py (OneStopEnglish Knowledge Features)
License: CC-BY-SA 4.0

Original Author: Bruce W. Lee (이웅성) @brucewlee
Affiliation 1: LXPER AI, Seoul, South Korea
Affiliation 2: University of Pennsylvania, PA, USA
Contributing Author: -
Affiliation : -
"""

import gensim
from lingfeat.utils import division

def richness(probability_list):
    sum_ = 0
    for idx, number in enumerate(probability_list):
        sum_ += float((idx+1)*number)
    return sum_


def clarity(probability_list):
    sum_ = 0
    for number in probability_list[1:]:
        sum_ += (probability_list[0] - number)
    result = division(sum_,len(probability_list))
    if len(probability_list) == 1:
        result = probability_list[0]
    return result


def noise(probability_list):
    mean = float(1.0/len(probability_list))
    s2 = 0
    s4 = 0
    for number in probability_list:
        s2 += (number - mean)**2
        s4 += (number - mean)**4

    result = len(probability_list)*(division(s4,(s2)**2))

    return result


def get_probability_lists(token_list, dir_path):
    n_topic_list = [50, 100, 150, 200]

    # lda_list is a list of Gensim lda objects
    lda_list = []

    # probability_lists is a list of list. 
    # [[probability list at topic 50],[probability list at topic 100],[probability list at topic 150],[probability list at topic 200]]
    probability_lists = []

    common_dictionary = gensim.corpora.dictionary.Dictionary([token_list])
    common_corpus = [common_dictionary.doc2bow(token_list)]

    for n_topic in n_topic_list:
        lda_list.append(gensim.models.ldamodel.LdaModel.load(dir_path+"/_AdvancedSemantic/model/onestop" + str(n_topic)))

    probability_list = []
    for lda in lda_list:
        for topic_probability in lda.get_document_topics(common_corpus)[0]:
            probability_list.append(topic_probability[1])
            probability_list = sorted(probability_list, reverse=True)
        probability_lists.append(probability_list)
    return probability_lists


def retrieve(token_list, dir_path):
    n_topic_list_for_naming = ["05", "10", "15", "20"]
    probability_lists = get_probability_lists(token_list, dir_path)

    # obtain each feature list
    richness_list = []
    clarity_list = []
    noise_list = []
    n_topics_list = []
    for probability_list in probability_lists:
        richness_list.append(richness(probability_list))
        clarity_list.append(clarity(probability_list))
        noise_list.append(noise(probability_list))
        n_topics_list.append(len(probability_list))
    
    # make resulting dictionary
    result = {}
    for i,feature in enumerate(richness_list):
        result["BRich"+n_topic_list_for_naming[i]+"_S"] = feature
    for i,feature in enumerate(clarity_list):
        result["BClar"+n_topic_list_for_naming[i]+"_S"] = feature
    for i,feature in enumerate(noise_list):
        result["BNois"+n_topic_list_for_naming[i]+"_S"] = feature
    for i,feature in enumerate(n_topics_list):
        result["BTopc"+n_topic_list_for_naming[i]+"_S"] = feature
    
    return result


# -*- coding: UTF-8 -*-
"""
Software: LingFeat - Comprehensive Linguistic Features for Readability Assessment
Page: ShaF.py (Shallow Features)
License: CC-BY-SA 4.0

Original Author: Bruce W. Lee (이웅성) @brucewlee
Affiliation 1: LXPER AI, Seoul, South Korea
Affiliation 2: University of Pennsylvania, PA, USA
Contributing Author: -
Affiliation : -


References:
>>> Shallow features inspired by 
Publication 1: Feng, Lijun, Noémie Elhadad, and Matt Huenerfauth. "Cognitively motivated features for readability assessment." Proceedings of the 12th Conference of the European Chapter of the ACL (EACL 2009). 2009.
"""
import math
from lingfeat.utils import count_syllables

def retrieve(origin_doc, token_list, n_token, n_sent):
    total_count_char = len(origin_doc.replace(" ",""))
    total_count_tokn = n_token
    total_count_syll = 0
    for token in token_list:
        total_count_syll += count_syllables(token)
    result = {
        "TokSenM_S":float(n_token*n_sent),
        "TokSenS_S":float(math.sqrt(n_token*n_sent)),
        "TokSenL_S":float(math.log(n_token)/math.log(n_sent)),
        "as_Token_C":float(total_count_tokn/n_sent),
        "as_Sylla_C":float(total_count_syll/n_sent),  
        "at_Sylla_C":float(total_count_syll/n_token),       
        "as_Chara_C":float(total_count_char/n_sent),   
        "at_Chara_C":float(total_count_char/n_token),
    }
    return result


# -*- coding: UTF-8 -*-
"""
Software: LingFeat - Comprehensive Linguistic Features for Readability Assessment
Page: TTRF.py (Type-Token-Ratio Features)
License: CC-BY-SA 4.0

Original Author: Bruce W. Lee (이웅성) @brucewlee
Affiliation 1: LXPER AI, Seoul, South Korea
Affiliation 2: University of Pennsylvania, PA, USA
Contributing Author: -
Affiliation : -


References:
>>> Entity Density features inspired by 
Publication 1: Malvern, David, and Brian Richards. "Measures of lexical richness." The encyclopedia of applied linguistics (2012).
"""
import math
from lingfeat.utils import division
    
def retrieve(n_token,token_list):
    n_utoken = 1
    default_MTLD = 0.72
    MTLD_count = 0
    for token in token_list:
        if token_list.count(token) == 1:
            n_utoken += 1
        if float(n_utoken/n_token) >= 0.72:
            MTLD_count += 1

    result={
        "SimpTTR_S":float(n_utoken/n_token),
        "CorrTTR_S":float(n_utoken/math.sqrt(2*n_token)),
        "BiLoTTR_S":float(math.log(n_utoken)/math.log(n_token)),
        "UberTTR_S":float(division(((math.log(n_utoken))**2),(math.log(n_token/n_utoken)))),
        "MTLDTTR_S":float(MTLD_count)
    }
    return result


# -*- coding: UTF-8 -*-
"""
Software: LingFeat - Comprehensive Linguistic Features for Readability Assessment
Page: TraF.py (Traditional Formulas Features)
License: CC-BY-SA 4.0

Original Author: Bruce W. Lee (이웅성) @brucewlee
Affiliation 1: LXPER AI, Seoul, South Korea
Affiliation 2: University of Pennsylvania, PA, USA
Contributing Author: -
Affiliation : -


References:
>>> Shallow features inspired by 
Publication 1: Kincaid, J. Peter, Robert P. Fishburne Jr, Richard L. Rogers, and Brad S. Chissom. Derivation of new readability formulas (automated readability index, fog count and flesch reading ease formula) for navy enlisted personnel. Naval Technical Training Command Millington TN Research Branch, 1975.
Publication 2: Luo Si and Jamie Callan. 2001. A statistical model for scientific readability. In Proceedings of the tenth international conference on Information and knowledge management (CIKM '01). Association for Computing Machinery, New York, NY, USA, 574–576.
Publication 3: Eltorai, Adam EM, Syed S. Naqvi, Soha Ghanian, Craig P. Eberson, Arnold‐Peter C. Weiss, Christopher T. Born, and Alan H. Daniels. "Readability of invasive procedure consent forms." Clinical and translational science 8, no. 6 (2015): 830-833.
"""
import math
from lingfeat.utils import count_syllables

class formulas:
    def __init__(self, origin_doc, sent_token_list, n_sent, n_token):
        self.n_sent = n_sent
        self.n_token = n_token
        n_syll = 0
        n_char = 0
        n_diff_token = 0
        n_poly_token = 0
        SMOG_upper_bound = len(sent_token_list)-10
        SMOG_middle_bound1 = len(sent_token_list)/2 - 5
        SMOG_middle_bound2 = len(sent_token_list)/2 + 5
        for i,sent in enumerate(sent_token_list):
            for token in sent:
                syll_this_token = count_syllables(token)
                n_syll += syll_this_token
                if syll_this_token >= 3:
                    n_diff_token += 1
                if syll_this_token >= 2:
                    if n_sent <= 30:
                        n_poly_token += 1
                    else:
                        if i <= 10 or i >= SMOG_upper_bound or SMOG_middle_bound1 <= i <= SMOG_middle_bound2:
                            n_poly_token += 1
                n_char += len(token)

        self.n_syll = n_syll
        self.n_char = n_char
        self.n_diff_token = n_diff_token
        self.n_easy_token = n_token - n_diff_token
        self.n_poly_token = n_poly_token

    # follow "new" automated readability index in reference 1
    def automated_readability_index(self):
        result = 0.37*(self.n_token/self.n_sent)+5.84*(self.n_char/self.n_token)-26.01
        return result
    
    # follow "new" fog count in reference 1
    def fog_count(self):
        result = ((self.n_easy_token + 3*(self.n_diff_token))/self.n_sent - 3)/2
        return result

    """
    # follow "old" flesch reading ease in reference 1
    def flesch_reading_ease(self):
        result = 206.835 - 1.015(self.n_token/self.n_sent) - 0.836(self.n_syll/(self.n_token/100))
        return result
    """

    # follow "new" flesch reading ease in reference 1
    def flesch_grade_level(self):
        result = 0.39*(self.n_token/self.n_sent) + 11.8*(self.n_syll/self.n_token) - 15.59
        return result

    # follow reference 2
    def smog_index(self):
        result = math.sqrt(self.n_poly_token)
        return result
    
    # follow reference 3
    def coleman_liau_index(self):
        result = 0.0588*(self.n_token/100) - 0.00296*(self.n_sent) - 15.8
        return result
    
    # follow reference 3
    def linsear_write_formula(self):
        result = (self.n_easy_token + 3*(self.n_diff_token))/self.n_sent
        if result > 10:
            result /= 2
        else:
            result -= 2
            result /= 2
        return result
    

def retrieve(origin_doc, sent_token_list, n_sent, n_token):
    Formulas = formulas(origin_doc, sent_token_list, n_sent, n_token)
    SMI = Formulas.smog_index()
    CML = Formulas.coleman_liau_index()
    GNF = Formulas.fog_count()
    ARI = Formulas.automated_readability_index()
    FKG = Formulas.flesch_grade_level()
    LWF = Formulas.linsear_write_formula()
    result = {
        "FleschG_S":FKG,
        "AutoRea_S":ARI,
        "ColeLia_S":CML,
        "SmogInd_S":SMI,
        "Gunning_S":GNF,
        "LinseaW_S":LWF,
    }
    return result


# -*- coding: UTF-8 -*-
"""
Software: LingFeat - Comprehensive Linguistic Features for Readability Assessment
Page: EnDF.py (Entity Density Features)
License: CC-BY-SA 4.0

Original Author: Bruce W. Lee (이웅성) @brucewlee
Affiliation 1: LXPER AI, Seoul, South Korea
Affiliation 2: University of Pennsylvania, PA, USA
Contributing Author: -
Affiliation : -

References:
>>> Entity Density features inspired by 
Publication 1: Feng, Lijun, Noémie Elhadad, and Matt Huenerfauth. "Cognitively motivated features for readability assessment." Proceedings of the 12th Conference of the European Chapter of the ACL (EACL 2009). 2009.
"""

def retrieve(NLP_doc, n_sent, n_token):
        to_EntiM_C = 0
        to_UEnti_C = 0
        ent_list = []
        unique_ent_list = []

        for ent in NLP_doc.ents:
            to_EntiM_C += 1
            ent_list.append(ent.text)
        
        for ent in ent_list:
            if ent_list.count(ent) == 1:
                to_UEnti_C += 1
                unique_ent_list.append(ent)

        result = {
            "to_EntiM_C": to_EntiM_C, 
            "as_EntiM_C": to_EntiM_C/n_sent, 
            "at_EntiM_C": to_EntiM_C/n_token,
            "to_UEnti_C": to_UEnti_C, 
            "as_UEnti_C": to_UEnti_C/n_sent,
            "at_UEnti_C": to_UEnti_C/n_token,
        }

        return result


# -*- coding: UTF-8 -*-
"""
Software: LingFeat - Comprehensive Linguistic Features for Readability Assessment
Page: WorF.py (Word Frequency Features)
License: CC-BY-SA 4.0

Original Author: Bruce W. Lee (이웅성) @brucewlee
Affiliation 1: LXPER AI, Seoul, South Korea
Affiliation 2: University of Pennsylvania, PA, USA
Contributing Author: -
Affiliation : -


References:
>>> Word Frequency features inspired by 
Publication 1: Vajjala, Sowmya, and Detmar Meurers. "Readability-based sentence ranking for evaluating text simplification." (2015).
"""

import pandas as pd

def retrieve(token_list, n_token, n_sent, dir_path):
    to_SbFrQ_C = 0
    to_SbCDC_C = 0
    to_SbFrL_C = 0
    to_SbCDL_C = 0
    to_SbSBW_C = 0
    to_SbL1W_C = 0
    to_SbSBC_C = 0
    to_SbL1C_C = 0

    DB = pd.read_csv(dir_path+'/_LexicoSemantic/resources/SUBTLEXus.csv')
    DB.set_index('Word_lowercased', inplace=True, drop=True)
    for token in token_list:
        if token in DB.index:
            scores_for_this_token = list(DB.loc[token, :])
            for i, score in enumerate(scores_for_this_token):
                scores_for_this_token[i] = 0 if str(score) == 'none' else scores_for_this_token[i]
            to_SbFrQ_C += float(scores_for_this_token[1])
            to_SbCDC_C += float(scores_for_this_token[2])
            to_SbFrL_C += float(scores_for_this_token[3])
            to_SbCDL_C += float(scores_for_this_token[4])
            to_SbSBW_C += float(scores_for_this_token[5])
            to_SbL1W_C += float(scores_for_this_token[6])
            to_SbSBC_C += float(scores_for_this_token[7])
            to_SbL1C_C += float(scores_for_this_token[8])

    result = {
        "to_SbFrQ_C": to_SbFrQ_C,
        "as_SbFrQ_C": to_SbFrQ_C/n_sent ,
        "at_SbFrQ_C": to_SbFrQ_C/n_token,
        "to_SbCDC_C": to_SbCDC_C,
        "as_SbCDC_C": to_SbCDC_C/n_sent ,
        "at_SbCDC_C": to_SbCDC_C/n_token,
        "to_SbFrL_C": to_SbFrL_C,
        "as_SbFrL_C": to_SbFrL_C/n_sent ,
        "at_SbFrL_C": to_SbFrL_C/n_token,
        "to_SbCDL_C": to_SbCDL_C,
        "as_SbCDL_C": to_SbCDL_C/n_sent ,
        "at_SbCDL_C": to_SbCDL_C/n_token,
        "to_SbSBW_C": to_SbSBW_C,
        "as_SbSBW_C": to_SbSBW_C/n_sent ,
        "at_SbSBW_C": to_SbSBW_C/n_token,
        "to_SbL1W_C": to_SbL1W_C,
        "as_SbL1W_C": to_SbL1W_C/n_sent ,
        "at_SbL1W_C": to_SbL1W_C/n_token,
        "to_SbSBC_C": to_SbSBC_C,
        "as_SbSBC_C": to_SbSBC_C/n_sent ,
        "at_SbSBC_C": to_SbSBC_C/n_token,
        "to_SbL1C_C": to_SbL1C_C,
        "as_SbL1C_C": to_SbL1C_C/n_sent ,
        "at_SbL1C_C": to_SbL1C_C/n_token,
    }
    return result  


# -*- coding: UTF-8 -*-
"""
Software: LingFeat - Comprehensive Linguistic Features for Readability Assessment
Page: VarF.py (Variational Features)
License: CC-BY-SA 4.0

Original Author: Bruce W. Lee (이웅성) @brucewlee
Affiliation 1: LXPER AI, Seoul, South Korea
Affiliation 2: University of Pennsylvania, PA, USA
Contributing Author: -
Affiliation : -


References:
>>> Variational features inspired by 
Publication 1: Lu, Xiaofei. "A corpus‐based evaluation of syntactic complexity measures as indices of college‐level ESL writers' language development." TESOL quarterly 45.1 (2011): 36-62.
Publication 2: Vajjala, Sowmya, and Detmar Meurers. "On improving the accuracy of readability classification using insights from second language acquisition." Proceedings of the seventh workshop on building educational applications using NLP. 2012.
"""
import math
from lingfeat.utils import division

def retrieve(NLP_doc):
    noun_list = []
    verb_list = []
    adje_list = []
    adve_list = []
    n_unoun = 0
    n_uverb = 0
    n_uadje = 0
    n_uadve = 0
    for token in NLP_doc:
        if token.pos_ == "NOUN":
            noun_list.append(token.lemma)
        if token.pos_ == "VERB":
            verb_list.append(token.lemma)
        if token.pos_ == "ADJ":
            adje_list.append(token.lemma)
        if token.pos_ == "ADV":
            adve_list.append(token.lemma)
    for noun in noun_list:
        if noun_list.count(noun) == 1:
            n_unoun += 1
    for verb in verb_list:
        if verb_list.count(verb) == 1:
            n_uverb += 1
    for adje in adje_list:
        if adje_list.count(adje) == 1:
            n_uadje += 1
    for adve in adve_list:
        if adve_list.count(adve) == 1:
            n_uadve += 1
    
    result = {
        "SimpNoV_S":float(division(n_unoun,(len(noun_list)))),
        "SquaNoV_S":float(division((n_unoun)**2,(len(noun_list)))),
        "CorrNoV_S":float(division(n_unoun,(math.sqrt(2*len(noun_list))))),
        "SimpVeV_S":float(division(n_uverb,(len(verb_list)))),
        "SquaVeV_S":float(division((n_uverb)**2,(len(verb_list)))),
        "CorrVeV_S":float(division(n_uverb,(math.sqrt(2*len(verb_list))))),
        "SimpAjV_S":float(division(n_uadje,(len(adje_list)))),
        "SquaAjV_S":float(division((n_uadje)**2,(len(adje_list)))),
        "CorrAjV_S":float(division(n_uadje,(math.sqrt(2*len(adje_list))))),
        "SimpAvV_S":float(division(n_uadve,(len(adve_list)))),
        "SquaAvV_S":float(division((n_uadve)**2,(len(adve_list)))),
        "CorrAvV_S":float(division(n_uadve,(math.sqrt(2*len(adve_list))))),
    }

    return result